package mscJNeuralNet.connectors;

import java.util.ResourceBundle;

import observerPattern.Observable;

import formats.Formats;

import logger.Log;
import mscJNeuralNet.net.Net;

/**
 * Kantengewichtsinitialisierung mit Zufallswerten entsprechend den eingehenden Verbindungen
 * eines Neurons nach Wessels und Barnard.
 * <p/>
 * Aus <i>Wessels and Barnard, 1992</i>: "the initial weight of a unit i should be
 * in the order of 1/sqrt(Fi) where Fi is the amount of incoming connections to the
 * unit i."
 * <p/>
 * Im folgenden Beispiel wird der RandomSymmetryBreakingNetConnector benutzt, 
 * um die Kantengewichte mit Zufallswerten entsprechend zu initialisieren:<br/>
 * <pre>
 * int [] lLayerSizes = {2, 1}; 
 * Net myNet = new Net();
 * INetConnector lWBSymBreakCon = new WesselsBarnardNetConnector();
 * lWBSymBreakCon.connectNet(myNet, null); 
 * <p/>
 * Created on 02.06.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 02.06.2004
 * @see INetConnector
 */
public class WesselsBarnardNetConnector implements INetConnector {
	private static final ResourceBundle smResourceBundle = ResourceBundle.getBundle("debug");
	private static final boolean smLogMethodCall;
	static{
		if (smResourceBundle.getString("showMethodCall.NetConnector").equalsIgnoreCase("true"))
			smLogMethodCall = true;
		else smLogMethodCall = false;
	}
	
	/**
	 * Gegebene Parameter werden nicht bercksichtigt. Daher sollte der 
	 * Parameterarray als null bergeben werden.<br/>
	 * Nach der Initialisierung werden die Observer des Netzes benachrichtigt.
	 * 
	 * @see mscJNeuralNet.connectors.INetConnector#connectNet(Net, double[])
	 * @see observerPattern.ObserverManager#notifyObservers(Observable)
	 */
	public void connectNet(Net pNet, double [] pParameters) {
		double[][] lWeights = pNet.getWeights();
		int [][] lLayerNeurons = pNet.getLayerNeurons();
		if (smLogMethodCall) Log.getInstance().writeln(this.getClass().getName()+".createConnections("+Formats.format(pParameters)+")");
		
		// weightmatrix entsprechend mLayerNeurons erzeugen
		
		int lInputtingLayer, lOutputtingLayer;
		int lInputtingNeuronIndex, lOutputtingNeuronIndex;
		
		// first connect BIAS to all but inputlayer
		for (lInputtingLayer = 1; lInputtingLayer<lLayerNeurons.length; lInputtingLayer++){
			for (lInputtingNeuronIndex=0; lInputtingNeuronIndex<lLayerNeurons[lInputtingLayer].length; lInputtingNeuronIndex++){
				lWeights
					[0]
					[lLayerNeurons[lInputtingLayer][lInputtingNeuronIndex]] = Net.getRandomWeightValue(1D/(Math.sqrt(lLayerNeurons[lInputtingLayer-1].length+1) ));
			}
		}

		for (lOutputtingLayer=0; lOutputtingLayer<lLayerNeurons.length-1; lOutputtingLayer++){
			lInputtingLayer = lOutputtingLayer+1;
			for (lOutputtingNeuronIndex=0; lOutputtingNeuronIndex<lLayerNeurons[lOutputtingLayer].length; lOutputtingNeuronIndex++){
				for (lInputtingNeuronIndex=0; lInputtingNeuronIndex<lLayerNeurons[lInputtingLayer].length; lInputtingNeuronIndex++){
					lWeights
						[lLayerNeurons[lOutputtingLayer][lOutputtingNeuronIndex]]
						[lLayerNeurons[lInputtingLayer][lInputtingNeuronIndex]] = Net.getRandomWeightValue(1D/(Math.sqrt(lLayerNeurons[lOutputtingLayer].length+1) ));
				}
			}
		}
		if (smResourceBundle.getString("showCreateConnections").equalsIgnoreCase("true"))
			Log.getInstance().writeln(Formats.format(lWeights));
			
		pNet.getObserverManager().notifyObservers(pNet);
	}

	public String toString(){	return "Wessels Barnard Connector";}


}
