package mscJNeuralNet.trainingAlgorithms;
import mscJNeuralNet.net.Net;
import observerPattern.Observable;

/**
 * Schnittstelle fr alle Lernverfahren.
 * <p/>
 * Created on 01.04.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 04.06.2004
 * @see mscJNeuralNet.trainingAlgorithms
 */
public interface INetTrainingAlgorithm extends Observable{

	/**
	 * Initialisiert das Lernverfahren.
	 * Alle Werte (Fehler, Gradienten), die von dem Lernverfahren
	 * benutzt werden, werden hier mit ihren initial Werten belegt.
	 * Lediglich die Lernparameter bleiben, wie sie sind.
	 * Der Lernschritt ({@link #setCycle(int)}, {@link #getCycle()})
	 * muss beim Aufruf dieser Methode auf den Wert 0 gesetzt werden.
	 */
	public void init();
	
	/**
	 * (Er-)setzt das Netz, das durch dieses Lernverfahren trainiert werden soll.
	 * Die Methode {@link #init()} wird am Ende aufgerufen, damit das
	 * Lernverfahren initialisiert wird.
	 * 
	 * @param pNet Das zu trainierende Netz.
	 */
	public void setNet(Net pNet);
	
	/**
	 * Liefert das Netz, welches derzeit von diesem Lernverfahren trainiert wird.
	 * 
	 * @return Das aktuelle Netz.
	 */
	public Net getNet();
	
	/**
	 * Fhrt die Aktualisierung des Gewichtes W(outputting,inputting)
	 * durch.
	 * Diese Methode wird einmal pro Lernschritt und Gewicht aufgerufen.
	 * 
	 * @param pOutputtingNeuron Der Index des Neurons, von dem das Gewicht ausgeht.
	 * @param pInputtingNeuron Der Index des Neurons, in die das Gewicht eingeht.
	 */
	public void applyWeightUpdate(int pOutputtingNeuron, int pInputtingNeuron);
	
	/**
	 * Legt die Lernparameter fr das Lernverfahren fest.
	 * Negative Werte bedeuten, dass derjenige Wert nicht angewendet werden soll.
	 * Jedes Lernverfahren hat seine eigenen Lernparameter mit verschiedenen 
	 * Funktionen und auch in verschiedener Anzahl.
	 * 
	 * @param pLearningParameters Lernparameter fr das Lernverfahren fest.
	 */
	public void setLearningParameters(double [] pLearningParameters);
	
	/**
	 * Liefert die aktuellen Lernparameter fr das Lernverfahren.
	 * Jedes Lernverfahren hat seine eigenen Lernparameter mit verschiedenen 
	 * Funktionen und auch in verschiedener Anzahl.
	 * 
	 * @return Aktuelle Lernparameter fr das Lernverfahren.
	 */
	public double [] getLearningParameters();
	
	/**
	 * Liefert die Initialwerte fr die Lernparameter des Lernverfahrens.
	 * Jedes Lernverfahren hat seine eigenen Lernparameter mit verschiedenen 
	 * Funktionen und auch in verschiedener Anzahl.
	 * 
	 * @return Initialwerte fr die Lernparameter des Lernverfahrens.
	 */
	public double [] getDefaultLearningParameters();
	
	/**
	 * Liefert die Bezeichnungen fr die Lernparameter.
	 * Jedes Lernverfahren hat seine eigenen Lernparameter mit verschiedenen 
	 * Funktionen und auch in verschiedener Anzahl.
	 * 
	 * @return Die Bezeichnungen fr die Lernparameter.
	 */
	public String[] getLearningParameterNames();
	
	/**
	 * Liefert den Namen des Lernverfahrens. Der Name des Lernverfahrens 
	 * sollte eindeutig sein.
	 * 
	 * @return Namen des Lernverfahrens.
	 */
	public String getAlgorithmName();
	
	/**
	 * Legt den aktuellen Lernschritt fest.
	 * 
	 * @param pCycle Der aktuelle Lernschritt.
	 * @see #getCycle()
	 */
	public void setCycle(int pCycle);
	
	/**
	 * Liefert den aktuellen Lernschritt des Lernverfahrens.
	 * 
	 * @return Aktueller Lernschritt des Lernverfahrens.
	 * @see #setCycle(int)
	 */
	public int getCycle();
	
	/**
	 * Legt die aktuellen Gradienten fr die Gewichte fest.
	 * Diese Methode wird von der Klasse {@link Net} benutzt,
	 * um dem Lernverfahren bei jedem Lernschritt die aktuellen
	 * Gradienten mitzuteilen. 
	 * 
	 * @param pGradients Die aktuellen Gradienten fr die Gewichte.
	 */
	public void setActualGradients(double[][] pGradients);
	
	/**
	 * Liefert die aktuellen Gradienten fr die Gewichte.
	 * 
	 * @return Die aktuellen Gradienten fr die Gewichte.
	 * @see #setActualGradients(double[][])
	 */
	public double[][] getActualGradients();
}
